﻿using System;
using System.Text;
using System.Reflection;
using PaintDotNet;
using PaintDotNet.Effects;
using PaintDotNet.IndirectUI;
using PaintDotNet.PropertySystem;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Text;
//using System.Runtime.CompilerServices;
using System.Runtime.InteropServices;

[assembly: AssemblyTitle("MattsEffects")]
[assembly: AssemblyDescription("Some Effects")]
[assembly: AssemblyConfiguration("")]
[assembly: AssemblyCompany("BoltBait")]
[assembly: AssemblyProduct("BurninatePlugin")]
[assembly: AssemblyCopyright("Copyright © Matthew")]
[assembly: AssemblyTrademark("")]
[assembly: AssemblyCulture("")]
[assembly: ComVisible(false)]
[assembly: AssemblyVersion("1.0.*")]

namespace Matthew
{
    public class PluginSupportInfo : IPluginSupportInfo
    {
        public string Author
        {
            get
            {
                return "Matthew";
            }
        }
        public string Copyright
        {
            get
            {
                return ((AssemblyCopyrightAttribute)base.GetType().Assembly.GetCustomAttributes(typeof(AssemblyCopyrightAttribute), false)[0]).Copyright;
            }
        }

        public string DisplayName
        {
            get
            {
                return ((AssemblyProductAttribute)base.GetType().Assembly.GetCustomAttributes(typeof(AssemblyProductAttribute), false)[0]).Product;
            }
        }

        public Version Version
        {
            get
            {
                return base.GetType().Assembly.GetName().Version;
            }
        }

        public Uri WebsiteUri
        {
            get
            {
                return new Uri("http://www.google.com");
            }
        }
    }


    [PluginSupportInfo(typeof(PluginSupportInfo), DisplayName = "MattEffect")]
    public class MattEffectPlugin : PropertyBasedEffect
    {
        public static string StaticName
        {
            get
            {
                return "MattEffect";
            }
        }

        public static Image StaticIcon
        {
            get
            {
                Bitmap bm = null;
                Assembly a = Assembly.GetExecutingAssembly();

                using (System.IO.Stream bmStream = a.GetManifestResourceStream("Matthew.icon.png"))
                {
                    bm = new Bitmap(bmStream);
                }
                return bm;
            }
        }

        public MattEffectPlugin()
            : base(StaticName, StaticIcon, SubmenuNames.Render, EffectFlags.Configurable | EffectFlags.SingleThreaded)
        {
        }

        public enum PropertyNames
        {
            Amount1,
            Amount2,
            Amount3,
            Amount4
        }

        protected override PropertyCollection OnCreatePropertyCollection()
        {
            List<Property> props = new List<Property>();

            props.Add(new DoubleProperty(PropertyNames.Amount1, Math.PI / 2.0, 0, Math.PI));
            props.Add(new Int32Property(PropertyNames.Amount2, 3, 0, 200));
            props.Add(new Int32Property(PropertyNames.Amount3, 3, 0, 200));
            props.Add(new BooleanProperty(PropertyNames.Amount4, false));

            return new PropertyCollection(props);
        }

        protected override ControlInfo OnCreateConfigUI(PropertyCollection props)
        {
            ControlInfo configUI = CreateDefaultConfigUI(props);

            configUI.SetPropertyControlValue(PropertyNames.Amount1, ControlInfoPropertyNames.DisplayName, "Angle");
            configUI.SetPropertyControlValue(PropertyNames.Amount2, ControlInfoPropertyNames.DisplayName, "X Distance");
            configUI.SetPropertyControlValue(PropertyNames.Amount3, ControlInfoPropertyNames.DisplayName, "Y Distance");

            configUI.SetPropertyControlValue(PropertyNames.Amount1, ControlInfoPropertyNames.SliderLargeChange, Math.PI / 10);
            configUI.SetPropertyControlValue(PropertyNames.Amount1, ControlInfoPropertyNames.SliderSmallChange, Math.PI / 50);
            
            configUI.SetPropertyControlValue(PropertyNames.Amount2, ControlInfoPropertyNames.UpDownIncrement, 1);
            configUI.SetPropertyControlValue(PropertyNames.Amount3, ControlInfoPropertyNames.UpDownIncrement, 1);

            configUI.SetPropertyControlValue(PropertyNames.Amount4, ControlInfoPropertyNames.DisplayName, string.Empty);
            configUI.SetPropertyControlValue(PropertyNames.Amount4, ControlInfoPropertyNames.Description, "Black BG");

            return configUI;
        }

        protected override void OnSetRenderInfo(PropertyBasedEffectConfigToken newToken, RenderArgs dstArgs, RenderArgs srcArgs)
        {
            this.Amount1_Theta = newToken.GetProperty<DoubleProperty>(PropertyNames.Amount1).Value;
            this.Amount2_DistanceX = newToken.GetProperty<Int32Property>(PropertyNames.Amount2).Value;
            this.Amount3_DistanceY = newToken.GetProperty<Int32Property>(PropertyNames.Amount3).Value;
            this.Amount4 = newToken.GetProperty<BooleanProperty>(PropertyNames.Amount4).Value;

            base.OnSetRenderInfo(newToken, dstArgs, srcArgs);
        }

        protected override void OnRender(Rectangle[] rois, int startIndex, int length)
        {
            if (length == 0) return;
            for (int i = startIndex; i < startIndex + length; ++i)
            {
                Render(DstArgs.Surface, SrcArgs.Surface, rois[i]);
            }
        }

        protected override void OnCustomizeConfigUIWindowProperties(PropertyCollection props)
        {
            // Change the effect's window title
            props[ControlInfoPropertyNames.WindowTitle].Value = "Matts Effect v1.0";
            base.OnCustomizeConfigUIWindowProperties(props);
        }

        double Amount1_Theta = 100; //
        int Amount2_DistanceX = 4; // 
        int Amount3_DistanceY = 4; // 
        bool Amount4 = false; // 
        void Render(Surface dst, Surface src, Rectangle rc)
        {
            int dsx,dsy;

            if (Amount4)
            {
                this.DstArgs.Graphics.FillRectangle(Brushes.Black, rc);
            }
            else this.DstArgs.Graphics.DrawImage(this.SrcArgs.Bitmap, rc);

            for (int y = rc.Top; y < rc.Bottom; ++y)
               {
                dsy = (int)(Math.Sin(Amount1_Theta * y) * Amount3_DistanceY) + y; 
                bool tb = (dsy >= rc.Top && dsy < rc.Bottom);
                    for (int x = rc.Left; x < rc.Right; ++x)
                    {
                        dsx = (int)(Math.Sin(Amount1_Theta * x) * Amount2_DistanceX) + x;
                        
                        //dsy = (int)(Math.Sin(Amount1 * x) * Amount3) + y;
                        
                        //dsy = (int)(Math.Sin(Amount1 * x) * Amount2) + y;
                        //if (dsy >= rc.Top && dsy < rc.Bottom) dst[x, y] = src[x, dsy];

                        bool ta = (dsx >= rc.Left && dsx < rc.Right);
                       

                        if (ta && tb)
                        {
                            ColorBgra a = src[x, dsy];
                            ColorBgra b = src[dsx, y];

                            dst[x, y] = a.GetIntensity() > b.GetIntensity() ? a : b;
                        }
                        else if (ta)
                            dst[x, y] = src[dsx, y];
                        else if (tb)
                            dst[x, y] = src[x, dsy];
 
                        /*
                        if (ta && tb)
                        {
                            dst[x, y] = PaintDotNet.ColorBgra.Blend(src[x, dsy], src[dsx, y], 0);
                        } 
                       else if(ta)
                            dst[x, y] = src[dsx, y];
                        else if (tb)
                            dst[x, y] = src[x, dsy];
                         */
                        

                        //if (dsx >= rc.Left && dsx < rc.Right)
                        {
                            
                            //src[dsx, y] = dst[x, y];

                            /*
                            if ((dst[dsx, y].Bgra & 0xFFFFF00) != 0)
                                dst[dsx, y] = PaintDotNet.ColorBgra.Blend(dst[dsx, y],src[x, y], 200);
                            else
                                dst[dsx, y] = src[x, y];
                             */

                            /*
                            if ((dst[x, y].Bgra & 0xFFFFF00) != 0)
                                dst[x, y] = PaintDotNet.ColorBgra.Blend(dst[x, y], src[dsx, y],  200);
                            else
                                dst[x, y] = src[dsx, y];
                             */

                            //dst[dsx,y] = src[x, y];
                            //dst[x, y] = src[dsx, y];
                        }

                        
                        //if (dsy >= rc.Top && dsy < rc.Bottom)
                        {
                            //src[x, dsy] = dst[x, y];

                            /*
                            if ((dst[x, dsy].Bgra & 0xFFFFF00) != 0)
                                dst[x, dsy] = PaintDotNet.ColorBgra.Blend(dst[x, dsy], src[x, y], 200);
                            else
                                dst[x, dsy] = src[x, y];
                             */
 
                            /*
                            if (dsy >= rc.Top && dsy < rc.Bottom)
                            {
                                //src[x, dsy] = dst[x, y];
                                if ((dst[x, y].Bgra & 0xFFFFF00) != 0)
                                    dst[x, y] = PaintDotNet.ColorBgra.Blend(dst[x, y], src[x, dsy], 200);
                                else
                                    dst[x, y] = src[x, dsy];
                            }
                             */

                            //dst[x, dsy] = src[x, y];
                        }


                        /*
                        //y(x) = function

                        dsy = (int)(Math.Sin(Amount1 * x) * Amount2) + y;
                        dsx = (int)(Math.Sin(Amount1 * x) * Amount3) + x;
                        if (dsx >= rc.Left && dsx < rc.Right && dsy >= rc.Top && dsy < rc.Bottom)
                        //if (dsy >= rc.Top && dsy < rc.Bottom)
                        {
                            //dst[x, dsy] = src[x, y];
                            //dst[dsx, dsy] = src[x, y];
                            dst[x, y] = src[dsx, dsy];
                        }
                         */
                         
                    }
                }
        }
    }
}